/**
 * The `Apply` class provides the `ap` which is used to apply a function to an argument under a type constructor.
 *
 * `Apply` can be used to lift functions of two or more arguments to work on values wrapped with the type constructor
 * `f`.
 *
 * Instances must satisfy the following law in addition to the `Functor` laws:
 *
 * 1. Associative composition: `fbc |> map(bc => ab => a => bc(ab(a))) |> ap(fab) <-> fbc |> ap(fab |> ap(fa))`
 *
 * Formally, `Apply` represents a strong lax semi-monoidal endofunctor.
 *
 * @example
 * import * as O from 'fp-ts/Option'
 * import { pipe } from 'fp-ts/function'
 *
 * const f = (a: string) => (b: number) => (c: boolean) => a + String(b) + String(c)
 * const fa: O.Option<string> = O.some('s')
 * const fb: O.Option<number> = O.some(1)
 * const fc: O.Option<boolean> = O.some(true)
 *
 * assert.deepStrictEqual(
 *   pipe(
 *     // lift a function
 *     O.some(f),
 *     // apply the first argument
 *     O.ap(fa),
 *     // apply the second argument
 *     O.ap(fb),
 *     // apply the third argument
 *     O.ap(fc)
 *   ),
 *   O.some('s1true')
 * )
 *
 * @since 3.0.0
 */
import { flow, pipe } from './function';
import * as SemigroupModule from './Semigroup';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `ap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const ap = (F, G) => (fa) => {
    return flow(F.map((gab) => (ga) => G.ap(ga)(gab)), F.ap(fa));
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const apFirst = (A) => (second) => flow(A.map((a) => () => a), A.ap(second));
/**
 * @category combinators
 * @since 3.0.0
 */
export const apSecond = (A) => (second) => flow(A.map(() => (b) => b), A.ap(second));
/**
 * @category combinators
 * @since 3.0.0
 */
export const apS = (F) => (name, fb) => flow(F.map((a) => (b) => Object.assign({}, a, { [name]: b })), F.ap(fb));
/**
 * @category combinators
 * @since 3.0.0
 */
export const apT = (F) => (fb) => (fas) => pipe(fas, F.map((a) => (b) => [...a, b]), F.ap(fb));
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Lift a semigroup into 'F', the inner values are concatenated using the provided `Semigroup`.
 *
 * @since 3.0.0
 */
export const getApplySemigroup = (F) => (S) => {
    const f = SemigroupModule.reverse(S).concat;
    return {
        concat: (second) => (first) => pipe(first, F.map(f), F.ap(second))
    };
};
// -------------------------------------------------------------------------------------
// deprecated
// -------------------------------------------------------------------------------------
// TODO: sequenceT
// TODO: sequenceS
