import { pipe } from './function';
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `reduce` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const reduce = (F, G) => (b, f) => F.reduce(b, (b, ga) => pipe(ga, G.reduce(b, f)));
/**
 * `foldMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const foldMap = (F, G) => {
    return (M) => (f) => F.foldMap(M)(G.foldMap(M)(f));
};
/**
 * `reduceRight` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export const reduceRight = (F, G) => {
    return (b, f) => F.reduceRight(b, (ga, b) => G.reduceRight(b, f)(ga));
};
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Similar to 'reduce', but the result is encapsulated in a monad.
 *
 * Note: this function is not generally stack-safe, e.g., for monads which build up thunks a la `IO`.
 *
 * @example
 * import { reduceE } from 'fp-ts/Foldable'
 * import { Chain, some } from 'fp-ts/Option'
 * import { tree, Foldable } from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const t = tree(1, [tree(2, []), tree(3, []), tree(4, [])])
 * assert.deepStrictEqual(pipe(t, reduceE(Foldable)(Chain)(some(0), (b, a) => (a > 2 ? some(b + a) : some(b)))), some(7))
 *
 * @since 3.0.0
 */
export function reduceE(F) {
    return (M) => (mb, f) => F.reduce(mb, (mb, a) => pipe(mb, M.chain((b) => f(b, a))));
}
/**
 * Fold a data structure, accumulating values in some `Monoid`, combining adjacent elements using the specified separator.
 *
 * @example
 * import { intercalate } from 'fp-ts/Foldable'
 * import { Monoid } from 'fp-ts/string'
 * import * as T from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const t = T.tree('a', [T.tree('b', []), T.tree('c', []), T.tree('d', [])])
 * assert.strictEqual(pipe(t, intercalate(T.Foldable)(Monoid)('|')), 'a|b|c|d')
 *
 * @since 3.0.0
 */
export function intercalate(F) {
    return (M) => (sep) => (fm) => {
        const go = ([init, acc], m) => init ? [false, m] : [false, pipe(acc, M.concat(sep), M.concat(m))];
        return pipe(fm, F.reduce([true, M.empty], go))[1];
    };
}
/**
 * Transforms a `Foldable` into a read-only array.
 *
 * @example
 * import { toReadonlyArray } from 'fp-ts/Foldable'
 * import { Foldable, tree } from 'fp-ts/Tree'
 *
 * const t = tree(1, [tree(2, []), tree(3, []), tree(4, [])])
 * assert.deepStrictEqual(toReadonlyArray(Foldable)(t), [1, 2, 3, 4])
 *
 * @since 3.0.0
 */
export function toReadonlyArray(F) {
    return (fa) => pipe(fa, F.reduce([], (acc, a) => {
        acc.push(a);
        return acc;
    }));
}
