/**
 * The `FromEither` type class represents those data types which support typed errors.
 *
 * @since 3.0.0
 */
import * as ChainModule from './Chain';
import { pipe } from './function';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromOption = (F) => (onNone) => {
    const fromOption = _.fromOption(onNone);
    return (ma) => F.fromEither(fromOption(ma));
};
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromPredicateOrElse = (F) => (predicate, onFalse) => (b) => F.fromEither(predicate(b) ? _.right(b) : _.left(onFalse(b)));
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromRefinementOrElse = (F) => (refinement, onFalse) => (c) => F.fromEither(refinement(c) ? _.right(c) : _.left(onFalse(c)));
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromOptionKOrElse = (F) => {
    const fromOptionF = fromOption(F);
    return (onNone) => {
        const from = fromOptionF(onNone);
        return (f) => (...a) => {
            return from(f(...a));
        };
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainOptionKOrElse = (F, M) => {
    const fromOptionKOrElseF = fromOptionKOrElse(F);
    return (onNone) => {
        const from = fromOptionKOrElseF(onNone);
        return (f) => {
            const fromf = from(f);
            return (ma) => {
                return pipe(ma, M.chain(fromf));
            };
        };
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const fromEitherK = (F) => (f) => (...a) => F.fromEither(f(...a));
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainEitherK = (F, M) => {
    const fromEitherKF = fromEitherK(F);
    return (f) => (ma) => {
        return pipe(ma, M.chain(fromEitherKF(f)));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const chainFirstEitherK = (F, M) => {
    const chainFirstM = ChainModule.chainFirst(M);
    const fromEitherKF = fromEitherK(F);
    return (f) => {
        const fromEitherKFf = fromEitherKF(f);
        return chainFirstM((a) => fromEitherKFf(a));
    };
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const filterOrElse = (F, M) => (predicate, onFalse) => {
    return M.chain((b) => F.fromEither(predicate(b) ? _.right(b) : _.left(onFalse(b))));
};
/**
 * @category combinators
 * @since 3.0.0
 */
export const refineOrElse = (F, M) => (refinement, onFalse) => {
    return M.chain((c) => F.fromEither(refinement(c) ? _.right(c) : _.left(onFalse(c))));
};
// -------------------------------------------------------------------------------------
// interop
// -------------------------------------------------------------------------------------
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableOrElse = (F) => (onNullable) => (a) => {
    return F.fromEither(_.fromNullableOrElse(onNullable)(a));
};
/**
 * @category interop
 * @since 3.0.0
 */
export const fromNullableKOrElse = (F) => {
    const fromNullableOrElseF = fromNullableOrElse(F);
    return (onNullable) => {
        const fromNullable = fromNullableOrElseF(onNullable);
        return (f) => (...a) => {
            return fromNullable(f(...a));
        };
    };
};
/**
 * @category interop
 * @since 3.0.0
 */
export const chainNullableKOrElse = (F, M) => {
    const fromNullableKM = fromNullableKOrElse(F);
    return (onNullable) => {
        const fromNullable = fromNullableKM(onNullable);
        return (f) => (ma) => {
            return pipe(ma, M.chain(fromNullable(f)));
        };
    };
};
