/**
 * The `FromEither` type class represents those data types which support typed errors.
 *
 * @since 3.0.0
 */
import * as ChainModule from './Chain'
import type { Either } from './Either'
import { Lazy } from './function'
import type { HKT, Kind, Typeclass } from './HKT'
import type { Option } from './Option'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import Chain = ChainModule.Chain
/**
 * @category type classes
 * @since 3.0.0
 */
export interface FromEither<F extends HKT> extends Typeclass<F> {
  readonly fromEither: <E, A, S, R = unknown, W = never>(fa: Either<E, A>) => Kind<F, S, R, W, E, A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromOption: <F extends HKT>(
  F: FromEither<F>
) => <E>(onNone: Lazy<E>) => <A, S, R = unknown, W = never>(fa: Option<A>) => Kind<F, S, R, W, E, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicateOrElse: <F extends HKT>(
  F: FromEither<F>
) => <B extends A, E, A = B>(
  predicate: Predicate<A>,
  onFalse: (b: B) => E
) => <S, R = unknown, W = never>(b: B) => Kind<F, S, R, W, E, B>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromRefinementOrElse: <F extends HKT>(
  F: FromEither<F>
) => <C extends A, B extends A, E, A = C>(
  refinement: Refinement<A, B>,
  onFalse: (c: C) => E
) => <S, R = unknown, W = never>(c: C) => Kind<F, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionKOrElse: <F extends HKT>(
  F: FromEither<F>
) => <E>(
  onNone: Lazy<E>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => Option<B>
) => <S, R = unknown, W = never>(...a: A) => Kind<F, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainOptionKOrElse: <M extends HKT>(
  F: FromEither<M>,
  M: ChainModule.Chain<M>
) => <E>(
  onNone: Lazy<E>
) => <A, B>(f: (a: A) => Option<B>) => <S, R, W>(ma: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <F extends HKT>(
  F: FromEither<F>
) => <A extends readonly unknown[], E, B>(
  f: (...a: A) => Either<E, B>
) => <S, R = unknown, W = never>(...a: A) => Kind<F, S, R, W, E, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainEitherK: <M extends HKT>(
  F: FromEither<M>,
  M: ChainModule.Chain<M>
) => <A, E2, B>(
  f: (a: A) => Either<E2, B>
) => <S, R, W, E1>(ma: Kind<M, S, R, W, E1, A>) => Kind<M, S, R, W, E2 | E1, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstEitherK: <M extends HKT>(
  F: FromEither<M>,
  M: ChainModule.Chain<M>
) => <A, E2, B>(
  f: (a: A) => Either<E2, B>
) => <S, R, W, E1>(ma: Kind<M, S, R, W, E1, A>) => Kind<M, S, R, W, E2 | E1, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const filterOrElse: <M extends HKT>(
  F: FromEither<M>,
  M: ChainModule.Chain<M>
) => <B extends A, E2, A = B>(
  predicate: Predicate<A>,
  onFalse: (b: B) => E2
) => <S, R, W, E1>(mb: Kind<M, S, R, W, E1, B>) => Kind<M, S, R, W, E2 | E1, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const refineOrElse: <M extends HKT>(
  F: FromEither<M>,
  M: ChainModule.Chain<M>
) => <C extends A, B extends A, E2, A = C>(
  refinement: Refinement<A, B>,
  onFalse: (c: C) => E2
) => <S, R, W, E1>(ma: Kind<M, S, R, W, E1, C>) => Kind<M, S, R, W, E2 | E1, B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableOrElse: <F extends HKT>(
  F: FromEither<F>
) => <E>(onNullable: Lazy<E>) => <A, S, R = unknown, W = never>(a: A) => Kind<F, S, R, W, E, NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableKOrElse: <F extends HKT>(
  F: FromEither<F>
) => <E>(
  onNullable: Lazy<E>
) => <A extends readonly unknown[], B>(
  f: (...a: A) => B | null | undefined
) => <S, R = unknown, W = never>(...a: A) => Kind<F, S, R, W, E, NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const chainNullableKOrElse: <M extends HKT>(
  F: FromEither<M>,
  M: ChainModule.Chain<M>
) => <E>(
  onNullable: Lazy<E>
) => <A, B>(
  f: (a: A) => B | null | undefined
) => <S, R, W>(ma: Kind<M, S, R, W, E, A>) => Kind<M, S, R, W, E, NonNullable<B>>
