/**
 * ```ts
 * interface IO<A> {
 *   (): A
 * }
 * ```
 *
 * `IO<A>` represents a non-deterministic synchronous computation that can cause side effects, yields a value of
 * type `A` and **never fails**.
 *
 * If you want to represent a synchronous computation that may fail, please see `IOEither`.
 * If you want to represent a synchronous computation that may yield nothing, please see `IOOption`.
 *
 * @since 3.0.0
 */
import * as ApplyModule from './Apply'
import * as ApplicativeModule from './Applicative'
import * as ChainModule from './Chain'
import * as ChainRecModule from './ChainRec'
import type { Either } from './Either'
import * as FromIOModule from './FromIO'
import * as FunctorModule from './Functor'
import type { HKT } from './HKT'
import * as MonadModule from './Monad'
import * as PointedModule from './Pointed'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface IO<A> {
  (): A
}
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: IO<A>) => IO<B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: IO<A>) => <B>(fab: IO<(a: A) => B>) => IO<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => IO<A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, B>(f: (a: A) => IO<B>) => (ma: IO<A>) => IO<B>
/**
 * @category ChainRec
 * @since 3.0.0
 */
export declare const chainRec: <A, B>(f: (a: A) => IO<Either<A, B>>) => (a: A) => IO<B>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: IO<IO<A>>) => IO<A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface IOF extends HKT {
  readonly type: IO<this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: FunctorModule.Functor<IOF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: IO<(a: A) => B>) => IO<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: PointedModule.Pointed<IOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: ApplyModule.Apply<IOF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: IO<B>) => <A>(first: IO<A>) => IO<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: IO<B>) => <A>(first: IO<A>) => IO<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: ApplicativeModule.Applicative<IOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: ChainModule.Chain<IOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: MonadModule.Monad<IOF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => IO<B>) => (first: IO<A>) => IO<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: FromIOModule.FromIO<IOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ChainRec: ChainRecModule.ChainRec<IOF>
/**
 * @since 3.0.0
 */
export declare const Do: IO<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(fa: IO<A>) => IO<{
  readonly [K in N]: A
}>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (fa: IO<A>) => IO<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => IO<B>
) => (ma: IO<A>) => IO<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: IO<B>
) => (fa: IO<A>) => IO<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const ApT: IO<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: IO<A>) => IO<readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <B>(fb: IO<B>) => <A extends ReadonlyArray<unknown>>(fas: IO<A>) => IO<readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IO<B>
) => (as: ReadonlyNonEmptyArray<A>) => IO<ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IO<B>
) => (as: readonly A[]) => IO<readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, B>(
  f: (a: A) => IO<B>
) => (as: ReadonlyNonEmptyArray<A>) => IO<ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, B>(f: (a: A) => IO<B>) => (as: readonly A[]) => IO<readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <A>(arr: ReadonlyArray<IO<A>>) => IO<ReadonlyArray<A>>
