/**
 * `IOOption<A>` represents a synchronous computation that either yields a value of type `A` or nothing.
 *
 * If you want to represent a synchronous computation that never fails, please see `IO`.
 * If you want to represent a synchronous computation that may fail, please see `IOEither`.
 *
 * @since 3.0.0
 */
import * as AltModule from './Alt'
import * as AlternativeModule from './Alternative'
import * as ApplicativeModule from './Applicative'
import * as ApplyModule from './Apply'
import * as ChainModule from './Chain'
import * as CompactableModule from './Compactable'
import type { Either } from './Either'
import * as FilterableModule from './Filterable'
import * as FromOptionModule from './FromOption'
import * as FromEitherModule from './FromEither'
import * as FromIOModule from './FromIO'
import { Lazy } from './function'
import * as FunctorModule from './Functor'
import type { HKT } from './HKT'
import * as IOModule from './IO'
import type { IOEither } from './IOEither'
import * as MonadModule from './Monad'
import * as OptionModule from './Option'
import * as PointedModule from './Pointed'
import type { Predicate } from './Predicate'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Refinement } from './Refinement'
import type { Separated } from './Separated'
import * as ZeroModule from './Zero'
import IO = IOModule.IO
import Option = OptionModule.Option
/**
 * @category model
 * @since 3.0.0
 */
export interface IOOption<A> extends IO<Option<A>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const some: <A>(a: A) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <A>(fa: Option<A>) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromEither: <A>(e: Either<unknown, A>) => IOModule.IO<OptionModule.Option<A>>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIO: <A>(ma: IOModule.IO<A>) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: <A>(ma: IOEither<unknown, A>) => IOOption<A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <B, A, C = B>(onNone: () => B, onSome: (a: A) => C) => (ma: IOOption<A>) => IO<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchE: <B, A, C = B>(
  onNone: () => IO<B>,
  onSome: (a: A) => IO<C>
) => (ma: IOOption<A>) => IO<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElse: <B>(onNone: Lazy<B>) => <A>(ma: IOOption<A>) => IO<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElseE: <B>(onNone: Lazy<IO<B>>) => <A>(ma: IOOption<A>) => IO<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const toUndefined: <A>(ma: IOOption<A>) => IO<A | undefined>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const toNullable: <A>(ma: IOOption<A>) => IO<A | null>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: IOOption<A>) => IOOption<B>
/**
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: IOOption<A>) => <B>(fab: IOOption<(a: A) => B>) => IOOption<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => IOOption<A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, B>(f: (a: A) => IOOption<B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: IOOption<IOOption<A>>) => IOOption<A>
/**
 * @category Alt
 * @since 3.0.0
 */
export declare const alt: <B>(second: Lazy<IOOption<B>>) => <A>(first: IOOption<A>) => IOOption<A | B>
/**
 * @category Zero
 * @since 3.0.0
 */
export declare const zero: <A>() => IOOption<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const none: IOOption<never>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <A>(foa: IOOption<OptionModule.Option<A>>) => IOOption<A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <A, B>(fe: IOOption<Either<A, B>>) => Separated<IOOption<A>, IOOption<B>>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => Option<B>) => (fga: IOOption<A>) => IOOption<B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => (fa: IOOption<A>) => Separated<IOOption<B>, IOOption<C>>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface IOOptionF extends HKT {
  readonly type: IOOption<this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: FunctorModule.Functor<IOOptionF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: IOOption<(a: A) => B>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: PointedModule.Pointed<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: ApplyModule.Apply<IOOptionF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: IOOption<B>) => <A>(first: IOOption<A>) => IOOption<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: IOOption<B>) => <A>(first: IOOption<A>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: ApplicativeModule.Applicative<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: ChainModule.Chain<IOOptionF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => IOOption<B>) => (first: IOOption<A>) => IOOption<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alt: AltModule.Alt<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Zero: ZeroModule.Zero<IOOptionF>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const guard: (b: boolean) => IOOption<void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alternative: AlternativeModule.Alternative<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: MonadModule.Monad<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: CompactableModule.Compactable<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: FilterableModule.Filterable<IOOptionF>
/**
 * @since 3.0.0
 */
export declare const filter: <B extends A, A = B>(predicate: Predicate<A>) => (fb: IOOption<B>) => IOOption<B>
/**
 * @since 3.0.0
 */
export declare const refine: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => (fc: IOOption<C>) => IOOption<B>
/**
 * @since 3.0.0
 */
export declare const partition: <B extends A, A = B>(
  predicate: Predicate<A>
) => (fb: IOOption<B>) => Separated<IOOption<B>, IOOption<B>>
/**
 * @since 3.0.0
 */
export declare const refinement: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => (fc: IOOption<C>) => Separated<IOOption<C>, IOOption<B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: FromIOModule.FromIO<IOOptionF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => IOModule.IO<B>
) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainIOK: <A, B>(f: (a: A) => IOModule.IO<B>) => (first: IOOption<A>) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstIOK: <A, B>(f: (a: A) => IOModule.IO<B>) => (first: IOOption<A>) => IOOption<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromOption: FromOptionModule.FromOption<IOOptionF>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: <B extends A, A = B>(predicate: Predicate<A>) => (b: B) => IOOption<B>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromRefinement: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => (c: C) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => Option<B>
) => (...a: A) => IOOption<B>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <A>(a: A) => IOOption<NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => IOOption<NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const chainNullableK: <A, B>(
  f: (a: A) => B | null | undefined
) => (ma: IOOption<A>) => IOOption<NonNullable<B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: FromEitherModule.FromEither<IOOptionF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: IOOption<A>) => IOOption<A>
/**
 * @since 3.0.0
 */
export declare const Do: IOOption<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(fa: IOOption<A>) => IOOption<{
  readonly [K in N]: A
}>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (fa: IOOption<A>) => IOOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => IOOption<B>
) => (ma: IOOption<A>) => IOOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: IOOption<B>
) => (fa: IOOption<A>) => IOOption<{
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}>
/**
 * @since 3.0.0
 */
export declare const ApT: IOOption<readonly []>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IOOption<B>
) => (as: ReadonlyNonEmptyArray<A>) => IOOption<ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IOOption<B>
) => (as: readonly A[]) => IOOption<readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, B>(
  f: (a: A) => IOOption<B>
) => (as: ReadonlyNonEmptyArray<A>) => IOOption<ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, B>(
  f: (a: A) => IOOption<B>
) => (as: readonly A[]) => IOOption<readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <A>(arr: ReadonlyArray<IOOption<A>>) => IOOption<ReadonlyArray<A>>
