/**
 * @since 3.0.0
 */
import * as AltModule from './Alt'
import * as ApplyModule from './Apply'
import * as ApplicativeModule from './Applicative'
import * as ChainModule from './Chain'
import * as ChainRecModule from './ChainRec'
import * as ComonadModule from './Comonad'
import type { Either } from './Either'
import type { Eq } from './Eq'
import * as FoldableModule from './Foldable'
import * as FunctorModule from './Functor'
import type { HKT, Kind } from './HKT'
import * as MonadModule from './Monad'
import type { Monoid } from './Monoid'
import * as PointedModule from './Pointed'
import type { Show } from './Show'
import * as TraversableModule from './Traversable'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Identity<A> = A
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: Identity<A>) => Identity<B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: Identity<A>) => <B>(fab: Identity<(a: A) => B>) => Identity<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => Identity<A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, B>(f: (a: A) => Identity<B>) => (ma: Identity<A>) => Identity<B>
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: <A, B>(f: (wa: Identity<A>) => B) => (wa: Identity<A>) => B
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: <A>(wa: Identity<A>) => A
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <A>(ma: Identity<A>) => Identity<Identity<A>>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: Identity<Identity<A>>) => Identity<A>
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduce: <B, A>(b: B, f: (b: B, a: A) => B) => (fa: Identity<A>) => B
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const foldMap: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: Identity<A>) => M
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: <B, A>(b: B, f: (a: A, b: B) => B) => (fa: Identity<A>) => B
/**
 * @category Traversable
 * @since 3.0.0
 */
export declare const traverse: <F extends HKT>(
  F: ApplicativeModule.Applicative<F>
) => <A, S, R, W, E, B>(f: (a: A) => Kind<F, S, R, W, E, B>) => (ta: Identity<A>) => Kind<F, S, R, W, E, B>
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export declare const alt: <B>(second: () => Identity<B>) => <A>(first: Identity<A>) => Identity<A | B>
/**
 * @category ChainRec
 * @since 3.0.0
 */
export declare const chainRec: <A, B>(f: (a: A) => Identity<Either<A, B>>) => (a: A) => B
/**
 * @category HKT
 * @since 3.0.0
 */
export interface IdentityF extends HKT {
  readonly type: Identity<this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: Eq<A>) => Eq<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: FunctorModule.Functor<IdentityF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Identity<(a: A) => B>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: PointedModule.Pointed<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: ApplyModule.Apply<IdentityF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: Identity<B>) => <A>(first: Identity<A>) => Identity<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: Identity<B>) => <A>(first: Identity<A>) => Identity<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: ApplicativeModule.Applicative<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: ChainModule.Chain<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: MonadModule.Monad<IdentityF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => Identity<B>) => (first: Identity<A>) => Identity<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ChainRec: ChainRecModule.ChainRec<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: FoldableModule.Foldable<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: TraversableModule.Traversable<IdentityF>
/**
 * @since 3.0.0
 */
export declare const sequence: <F extends HKT>(
  F: ApplicativeModule.Applicative<F>
) => <S, R, W, E, A>(fas: Identity<Kind<F, S, R, W, E, A>>) => Kind<F, S, R, W, E, Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alt: AltModule.Alt<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: ComonadModule.Comonad<IdentityF>
/**
 * @since 3.0.0
 */
export declare const Do: Identity<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(fa: Identity<A>) => {
  readonly [K in N]: A
}
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (fa: A) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => Identity<B>
) => (ma: Identity<A>) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: Identity<B>
) => (fa: Identity<A>) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
/**
 * @since 3.0.0
 */
export declare const ApT: Identity<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: Identity<A>) => readonly [A]
/**
 * @since 3.0.0
 */
export declare const apT: <B>(fb: B) => <A extends ReadonlyArray<unknown>>(fas: A) => readonly [...A, B]
