/**
 * @since 3.0.0
 */
import type { Applicative } from './Applicative'
import * as CompactableModule from './Compactable'
import type { Either } from './Either'
import type { Endomorphism } from './Endomorphism'
import * as EqModule from './Eq'
import * as FilterableModule from './Filterable'
import * as FilterableWithIndexModule from './FilterableWithIndex'
import type { Foldable } from './Foldable'
import type { FoldableWithIndex } from './FoldableWithIndex'
import * as FunctorModule from './Functor'
import type { FunctorWithIndex } from './FunctorWithIndex'
import type { HKT, Kind } from './HKT'
import type { Magma } from './Magma'
import type { Monoid } from './Monoid'
import * as OptionModule from './Option'
import type { Ord } from './Ord'
import type { Semigroup } from './Semigroup'
import * as SeparatedModule from './Separated'
import type { Show } from './Show'
import type { Traversable } from './Traversable'
import type { TraversableWithIndex } from './TraversableWithIndex'
import type { Unfoldable } from './Unfoldable'
import * as WitherableModule from './Witherable'
import type { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import Eq = EqModule.Eq
import Option = OptionModule.Option
import Separated = SeparatedModule.Separated
/**
 * Create a `ReadonlyMap` from one key/value pair.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const singleton: <K, A>(k: K, a: A) => ReadonlyMap<K, A>
/**
 * Create a `ReadonlyRecord` from a `Foldable` collection of key/value pairs, using the
 * specified `Magma` to combine values for duplicate keys, and the specified `f` to map to key/value pairs.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function fromFoldable<F extends HKT>(
  F: Foldable<F>
): <K, B>(
  E: Eq<K>,
  M: Magma<B>
) => <A>(f: (a: A) => readonly [K, B]) => <S, R, W, E>(fka: Kind<F, S, R, W, E, A>) => ReadonlyMap<K, B>
/**
 * Insert an element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key already exists.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const insertAt: <K>(
  E: EqModule.Eq<K>
) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => OptionModule.Option<ReadonlyMap<K, A>>
/**
 * Insert or replace a key/value pair in a `ReadonlyMap`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const upsertAt: <K>(E: EqModule.Eq<K>) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * Change the element at the specified keys, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export declare const updateAt: <K>(
  E: EqModule.Eq<K>
) => <A>(k: K, a: A) => (m: ReadonlyMap<K, A>) => OptionModule.Option<ReadonlyMap<K, A>>
/**
 * Apply a function to the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @since 3.0.0
 */
export declare const modifyAt: <K>(
  E: EqModule.Eq<K>
) => <A>(k: K, f: Endomorphism<A>) => (m: ReadonlyMap<K, A>) => OptionModule.Option<ReadonlyMap<K, A>>
/**
 * Delete the element at the specified key, creating a new `ReadonlyMap`, or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const deleteAt: <K>(
  E: EqModule.Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => OptionModule.Option<ReadonlyMap<K, A>>
/**
 * Delete a key and value from a `ReadonlyMap`, returning the value as well as the subsequent `ReadonlyMap`,
 * or returning `None` if the key doesn't exist.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pop: <K>(
  E: EqModule.Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => OptionModule.Option<readonly [A, ReadonlyMap<K, A>]>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <K>(fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category FunctorWithIndex
 * @since 3.0.0
 */
export declare const mapWithIndex: <K, A, B>(f: (k: K, a: A) => B) => (m: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <K, A>(m: ReadonlyMap<K, OptionModule.Option<A>>) => ReadonlyMap<K, A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <K, A, B>(
  fa: ReadonlyMap<K, Either<A, B>>
) => SeparatedModule.Separated<ReadonlyMap<K, A>, ReadonlyMap<K, B>>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(
  f: (a: A) => OptionModule.Option<B>
) => <K>(fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => <K>(fa: ReadonlyMap<K, A>) => Separated<ReadonlyMap<K, B>, ReadonlyMap<K, C>>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare const filterMapWithIndex: <K, A, B>(
  f: (k: K, a: A) => OptionModule.Option<B>
) => (fa: ReadonlyMap<K, A>) => ReadonlyMap<K, B>
/**
 * @category FilterableWithIndex
 * @since 3.0.0
 */
export declare const partitionMapWithIndex: <K, A, B, C>(
  f: (k: K, a: A) => Either<B, C>
) => (fa: ReadonlyMap<K, A>) => SeparatedModule.Separated<ReadonlyMap<K, B>, ReadonlyMap<K, C>>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface ReadonlyMapF extends HKT {
  readonly type: ReadonlyMap<this['Invariant1'], this['Covariant1']>
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface ReadonlyMapFFixedK<K> extends HKT {
  readonly type: ReadonlyMap<K, this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <K, A>(SK: Show<K>, SA: Show<A>) => Show<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <K, A>(EK: EqModule.Eq<K>, EA: EqModule.Eq<A>) => EqModule.Eq<ReadonlyMap<K, A>>
/**
 * Get a `Monoid` instance for `ReadonlyMap` given a `Semigroup` instance for its values.
 *
 * @category instances
 * @since 3.0.0
 */
export declare const getMonoid: <K, A>(EK: EqModule.Eq<K>, SA: Semigroup<A>) => Monoid<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: FunctorModule.Functor<ReadonlyMapF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <K, B>(fab: ReadonlyMap<K, (a: A) => B>) => ReadonlyMap<K, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFunctorWithIndex: <K = never>() => FunctorWithIndex<ReadonlyMapFFixedK<K>, K>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: CompactableModule.Compactable<ReadonlyMapF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: FilterableModule.Filterable<ReadonlyMapF>
/**
 * @since 3.0.0
 */
export declare const filter: <B extends A, A = B>(
  predicate: Predicate<A>
) => <K>(fb: ReadonlyMap<K, B>) => ReadonlyMap<K, B>
/**
 * @since 3.0.0
 */
export declare const refine: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => <K>(fc: ReadonlyMap<K, C>) => ReadonlyMap<K, B>
/**
 * @since 3.0.0
 */
export declare const partition: <B extends A, A = B>(
  predicate: Predicate<A>
) => <K>(fb: ReadonlyMap<K, B>) => Separated<ReadonlyMap<K, B>, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const refinement: <C extends A, B extends A, A = C>(
  refinement: Refinement<A, B>
) => <K>(fc: ReadonlyMap<K, C>) => Separated<ReadonlyMap<K, C>, ReadonlyMap<K, B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFilterableWithIndex: <K = never>() => FilterableWithIndexModule.FilterableWithIndex<
  ReadonlyMapFFixedK<K>,
  K
>
/**
 * @since 3.0.0
 */
export declare const filterWithIndex: <K, B extends A, A = B>(
  predicate: (i: K, a: A) => boolean
) => (fb: ReadonlyMap<K, B>) => ReadonlyMap<K, B>
/**
 * @since 3.0.0
 */
export declare const refineWithIndex: <K, C extends A, B extends A, A = C>(
  refinement: (i: K, a: A) => a is B
) => (fc: ReadonlyMap<K, C>) => ReadonlyMap<K, B>
/**
 * @since 3.0.0
 */
export declare const partitionWithIndex: <K, B extends A, A = B>(
  predicate: (i: K, a: A) => boolean
) => (fb: ReadonlyMap<K, B>) => Separated<ReadonlyMap<K, B>, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const refinementWithIndex: <K, C extends A, B extends A, A = C>(
  refinement: (i: K, a: A) => a is B
) => (fb: ReadonlyMap<K, C>) => Separated<ReadonlyMap<K, C>, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const reduce: <K>(O: Ord<K>) => <B, A>(b: B, f: (b: B, a: A) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMap: <K>(O: Ord<K>) => <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: ReadonlyMap<K, A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRight: <K>(O: Ord<K>) => <B, A>(b: B, f: (a: A, b: B) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldable: <K>(O: Ord<K>) => Foldable<ReadonlyMapFFixedK<K>>
/**
 * @since 3.0.0
 */
export declare const reduceWithIndex: <K>(
  O: Ord<K>
) => <B, A>(b: B, f: (i: K, b: B, a: A) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @since 3.0.0
 */
export declare const foldMapWithIndex: <K>(
  O: Ord<K>
) => <M>(M: Monoid<M>) => <A>(f: (i: K, a: A) => M) => (fa: ReadonlyMap<K, A>) => M
/**
 * @since 3.0.0
 */
export declare const reduceRightWithIndex: <K>(
  O: Ord<K>
) => <B, A>(b: B, f: (i: K, a: A, b: B) => B) => (fa: ReadonlyMap<K, A>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getFoldableWithIndex: <K>(O: Ord<K>) => FoldableWithIndex<ReadonlyMapFFixedK<K>, K>
/**
 * @since 3.0.0
 */
export declare const traverse: <K>(
  O: Ord<K>
) => <F extends HKT>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<F, S, R, W, E, B>
) => (ta: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const sequence: <K>(
  O: Ord<K>
) => <F extends HKT>(
  F: Applicative<F>
) => <S, R, W, E, A>(ta: ReadonlyMap<K, Kind<F, S, R, W, E, A>>) => Kind<F, S, R, W, E, ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversable: <K>(O: Ord<K>) => Traversable<ReadonlyMapFFixedK<K>>
/**
 * @since 3.0.0
 */
export declare const traverseWithIndex: <K>(
  O: Ord<K>
) => <F extends HKT>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (i: K, a: A) => Kind<F, S, R, W, E, B>
) => (ta: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, ReadonlyMap<K, B>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getTraversableWithIndex: <K>(O: Ord<K>) => TraversableWithIndex<ReadonlyMapFFixedK<K>, K>
/**
 * @since 3.0.0
 */
export declare const wither: <K>(
  O: Ord<K>
) => <F extends HKT>(
  F: Applicative<F>
) => <A, S, R, W, E, B>(
  f: (a: A) => Kind<F, S, R, W, E, OptionModule.Option<B>>
) => (ta: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, ReadonlyMap<K, B>>
/**
 * @since 3.0.0
 */
export declare const wilt: <K>(
  O: Ord<K>
) => <F extends HKT>(
  F: Applicative<F>
) => <A, S, R, W, E, B, C>(
  f: (a: A) => Kind<F, S, R, W, E, Either<B, C>>
) => (wa: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, SeparatedModule.Separated<ReadonlyMap<K, B>, ReadonlyMap<K, C>>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getWitherable: <K>(O: Ord<K>) => WitherableModule.Witherable<ReadonlyMapFFixedK<K>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionSemigroup: <K, A>(E: EqModule.Eq<K>, S: Semigroup<A>) => Semigroup<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getUnionMonoid: <K, A>(E: EqModule.Eq<K>, S: Semigroup<A>) => Monoid<ReadonlyMap<K, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getIntersectionSemigroup: <K, A>(
  E: EqModule.Eq<K>,
  S: Semigroup<A>
) => Semigroup<ReadonlyMap<K, A>>
/**
 * @category combinator
 * @since 3.0.0
 */
export declare const getDifferenceMagma: <K>(E: EqModule.Eq<K>) => <A>() => Magma<ReadonlyMap<K, A>>
/**
 * Calculate the number of key/value pairs in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const size: <K, A>(d: ReadonlyMap<K, A>) => number
/**
 * Test whether or not a `ReadonlyMap` is empty.
 *
 * @since 3.0.0
 */
export declare const isEmpty: <K, A>(d: ReadonlyMap<K, A>) => boolean
/**
 * Test whether or not a key exists in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const member: <K>(E: EqModule.Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => boolean
/**
 * Tests whether a value is a member of a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: EqModule.Eq<A>) => (a: A) => <K>(m: ReadonlyMap<K, A>) => boolean
/**
 * Get a sorted `ReadonlyArray` of the keys contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const keys: <K>(O: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => readonly K[]
/**
 * Get a sorted `ReadonlyArray` of the values contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const values: <A>(O: Ord<A>) => <K>(m: ReadonlyMap<K, A>) => readonly A[]
/**
 * @since 3.0.0
 */
export declare const collect: <K>(O: Ord<K>) => <A, B>(f: (k: K, a: A) => B) => (m: ReadonlyMap<K, A>) => readonly B[]
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 * If the result is a `Some`, the existing key is also returned.
 *
 * @since 3.0.0
 */
export declare const lookupWithKey: <K>(
  E: EqModule.Eq<K>
) => (k: K) => <A>(m: ReadonlyMap<K, A>) => OptionModule.Option<readonly [K, A]>
/**
 * Lookup the value for a key in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const lookup: <K>(E: EqModule.Eq<K>) => (k: K) => <A>(m: ReadonlyMap<K, A>) => OptionModule.Option<A>
/**
 * Test whether or not one `ReadonlyMap` contains all of the keys and values contained in another `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const isSubmap: <K, A>(
  EK: EqModule.Eq<K>,
  SA: EqModule.Eq<A>
) => (second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => boolean
/**
 * An empty `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const empty: <K>() => ReadonlyMap<K, never>
/**
 * Get a sorted `ReadonlyArray` of the key/value pairs contained in a `ReadonlyMap`.
 *
 * @since 3.0.0
 */
export declare const toReadonlyArray: <K>(O: Ord<K>) => <A>(m: ReadonlyMap<K, A>) => readonly (readonly [K, A])[]
/**
 * Unfolds a `ReadonlyMap` into a data structure of key/value pairs.
 *
 * @since 3.0.0
 */
export declare function toUnfoldable<F extends HKT>(
  U: Unfoldable<F>
): <K>(o: Ord<K>) => <A, S, R, W, E>(d: ReadonlyMap<K, A>) => Kind<F, S, R, W, E, readonly [K, A]>
/**
 * @since 3.0.0
 */
export declare const union: <K, A>(
  E: EqModule.Eq<K>,
  M: Magma<A>
) => (second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @since 3.0.0
 */
export declare const intersection: <K, A>(
  E: EqModule.Eq<K>,
  M: Magma<A>
) => (second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
/**
 * @since 3.0.0
 */
export declare const difference: <K>(
  E: EqModule.Eq<K>
) => <A>(_second: ReadonlyMap<K, A>) => (first: ReadonlyMap<K, A>) => ReadonlyMap<K, A>
