/**
 * @since 3.0.0
 */
import type { Applicative as Applicative_ } from './Applicative'
import { Apply as Apply_ } from './Apply'
import { Chain as Chain_ } from './Chain'
import type { Endomorphism } from './Endomorphism'
import type { FromState as FromState_ } from './FromState'
import { Functor as Functor_ } from './Functor'
import type { HKT } from './HKT'
import type { Monad as Monad_ } from './Monad'
import type { Pointed as Pointed_ } from './Pointed'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface State<S, A> {
  (s: S): readonly [A, S]
}
/**
 * Get the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const get: <S>() => State<S, S>
/**
 * Set the state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const put: <S>(s: S) => State<S, void>
/**
 * Modify the state by applying a function to the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const modify: <S>(f: Endomorphism<S>) => State<S, void>
/**
 * Get a value which depends on the current state
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const gets: <S, A>(f: (s: S) => A) => State<S, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <S>(fa: State<S, A>) => State<S, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <S, A>(fa: State<S, A>) => <B>(fab: State<S, (a: A) => B>) => State<S, B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A, S>(a: A) => State<S, A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, S, B>(f: (a: A) => State<S, B>) => (ma: State<S, A>) => State<S, B>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <S, A>(mma: State<S, State<S, A>>) => State<S, A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface StateF extends HKT {
  readonly type: State<this['Invariant1'], this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<StateF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <S, B>(fab: State<S, (a: A) => B>) => State<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<StateF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply_<StateF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <S, B>(second: State<S, B>) => <A>(first: State<S, A>) => State<S, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <S, B>(second: State<S, B>) => <A>(first: State<S, A>) => State<S, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative_<StateF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain_<StateF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad_<StateF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, S, B>(f: (a: A) => State<S, B>) => (first: State<S, A>) => State<S, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromState: FromState_<StateF>
/**
 * Run a computation in the `State` monad, discarding the final state.
 *
 * @since 3.0.0
 */
export declare const evaluate: <S>(s: S) => <A>(ma: State<S, A>) => A
/**
 * Run a computation in the `State` monad discarding the result.
 *
 * @since 3.0.0
 */
export declare const execute: <S>(s: S) => <A>(ma: State<S, A>) => S
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <S, A>(
  fa: State<S, A>
) => State<
  S,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <S>(fa: State<S, A>) => State<
  S,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, S, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => State<S, B>
) => (ma: State<S, A>) => State<
  S,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, S, B>(
  name: Exclude<N, keyof A>,
  fb: State<S, B>
) => (fa: State<S, A>) => State<
  S,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const tupled: <S, A>(fa: State<S, A>) => State<S, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <S, B>(
  fb: State<S, B>
) => <A extends ReadonlyArray<unknown>>(fas: State<S, A>) => State<S, readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, S, B>(
  f: (index: number, a: A) => State<S, B>
) => (as: ReadonlyNonEmptyArray<A>) => State<S, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, S, B>(
  f: (index: number, a: A) => State<S, B>
) => (as: readonly A[]) => State<S, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(Apply)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <A, S, B>(
  f: (a: A) => State<S, B>
) => (as: ReadonlyNonEmptyArray<A>) => State<S, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverse(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <A, S, B>(
  f: (a: A) => State<S, B>
) => (as: readonly A[]) => State<S, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <S, A>(arr: ReadonlyArray<State<S, A>>) => State<S, ReadonlyArray<A>>
