/**
 * @since 3.0.0
 */
import * as ApplicativeModule from './Applicative'
import type { Apply } from './Apply'
import { Bifunctor as Bifunctor_ } from './Bifunctor'
import type { Chain } from './Chain'
import type { ChainRec } from './ChainRec'
import type { Comonad as Comonad_ } from './Comonad'
import type { Foldable as Foldable_ } from './Foldable'
import { Functor as Functor_ } from './Functor'
import type { HKT, Kind } from './HKT'
import type { Monad } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed } from './Pointed'
import * as ReadonlyNonEmptyArrayModule from './ReadonlyNonEmptyArray'
import type { Semigroup } from './Semigroup'
import type { Semigroupoid as Semigroupoid_ } from './Semigroupoid'
import * as TraversableModule from './Traversable'
import ReadonlyNonEmptyArray = ReadonlyNonEmptyArrayModule.ReadonlyNonEmptyArray
/**
 * @category model
 * @since 3.0.0
 */
export declare type Writer<W, A> = readonly [A, W]
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromIdentity: <W>(w: W) => <A>(a: A) => Writer<W, A>
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const tell: <W>(w: W) => Writer<W, void>
/**
 * @since 3.0.0
 */
export declare const fst: <W, A>(t: Writer<W, A>) => A
/**
 * @since 3.0.0
 */
export declare const snd: <W, A>(t: Writer<W, A>) => W
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export declare const evaluate: <W, A>(fa: Writer<W, A>) => A
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export declare const execute: <W, A>(fa: Writer<W, A>) => W
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const swap: <W, A>(t: Writer<W, A>) => Writer<A, W>
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const listen: <W, A>(fa: Writer<W, A>) => Writer<W, readonly [A, W]>
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const pass: <W, A>(fa: Writer<W, readonly [A, (w: W) => W]>) => Writer<W, A>
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const listens: <W, B>(f: (w: W) => B) => <A>(fa: Writer<W, A>) => Writer<W, readonly [A, B]>
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const censor: <W>(f: (w: W) => W) => <A>(fa: Writer<W, A>) => Writer<W, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <W>(fa: Writer<W, A>) => Writer<W, B>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const mapLeft: <W, X>(f: (w: W) => X) => <A>(fea: Writer<W, A>) => Writer<X, A>
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const bimap: <W, X, A, B>(mapSnd: (w: W) => X, mapFst: (a: A) => B) => (t: Writer<W, A>) => Writer<X, B>
/**
 * Maps a function over the first component of a `Writer`.
 *
 * Alias of [`map`](#map)
 *
 * @since 3.0.0
 */
export declare const mapFst: <A, B>(f: (a: A) => B) => <W>(fa: Writer<W, A>) => Writer<W, B>
/**
 * Maps a function over the second component of a `Writer`.
 *
 * Alias of [`mapLeft`](#mapleft)
 *
 * @since 3.0.0
 */
export declare const mapSnd: <W, X>(f: (w: W) => X) => <A>(fea: Writer<W, A>) => Writer<X, A>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const compose: <B, C>(bc: Writer<B, C>) => <A>(ab: Writer<A, B>) => Writer<A, C>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const extend: <W, A, B>(f: (wa: Writer<W, A>) => B) => (wa: Writer<W, A>) => Writer<W, B>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const extract: <W, A>(wa: Writer<W, A>) => A
/**
 * Derivable from `Extend`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export declare const duplicate: <W, A>(t: Writer<W, A>) => Writer<W, Writer<W, A>>
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const reduce: <B, A>(b: B, f: (b: B, a: A) => B) => <W>(fa: Writer<W, A>) => B
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const foldMap: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => <W>(fa: Writer<W, A>) => M
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const reduceRight: <B, A>(b: B, f: (a: A, b: B) => B) => <W>(fa: Writer<W, A>) => B
/**
 * @category type class operations
 * @since 3.0.0
 */
export declare const traverse: <F extends HKT>(
  F: Apply<F>
) => <A, S, R, FW, E, B>(
  f: (a: A) => Kind<F, S, R, FW, E, B>
) => <W>(t: Writer<W, A>) => Kind<F, S, R, FW, E, Writer<W, B>>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface WriterF extends HKT {
  readonly type: Writer<this['Covariant2'], this['Covariant1']>
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface WriterFContravariant extends HKT {
  readonly type: Writer<this['Contravariant1'], this['Covariant1']>
}
/**
 * @category HKT
 * @since 3.0.0
 */
export interface WriterFFixedW<W> extends HKT {
  readonly type: Writer<W, this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Bifunctor: Bifunctor_<WriterF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<WriterF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <W, B>(fab: Writer<W, (a: A) => B>) => Writer<W, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Semigroupoid: Semigroupoid_<WriterFContravariant>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad_<WriterF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: Foldable_<WriterF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: TraversableModule.Traversable<WriterF>
/**
 * @since 3.0.0
 */
export declare const sequence: <F extends HKT>(
  F: Apply<F>
) => <W, FS, FR, FW, FE, A>(fa: Writer<W, Kind<F, FS, FR, FW, FE, A>>) => Kind<F, FS, FR, FW, FE, Writer<W, A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getPointed: <W>(M: Monoid<W>) => Pointed<WriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApply: <W>(S: Semigroup<W>) => Apply<WriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getApplicative: <W>(M: Monoid<W>) => ApplicativeModule.Applicative<WriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getChain: <W>(S: Semigroup<W>) => Chain<WriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getMonad: <W>(M: Monoid<W>) => Monad<WriterFFixedW<W>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare function getChainRec<W>(M: Monoid<W>): ChainRec<WriterFFixedW<W>>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(getApply(M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <W>(
  S: Semigroup<W>
) => <A, B>(
  f: (index: number, a: A) => Writer<W, B>
) => (
  as: ReadonlyNonEmptyArrayModule.ReadonlyNonEmptyArray<A>
) => Writer<W, ReadonlyNonEmptyArrayModule.ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <W>(
  M: Monoid<W>
) => <A, B>(f: (index: number, a: A) => Writer<W, B>) => (as: readonly A[]) => Writer<W, readonly B[]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverse(getApply(S))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArray: <W>(
  S: Semigroup<W>
) => <A, B>(
  f: (a: A) => Writer<W, B>
) => (
  as: ReadonlyNonEmptyArrayModule.ReadonlyNonEmptyArray<A>
) => Writer<W, ReadonlyNonEmptyArrayModule.ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverse(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArray: <W>(
  M: Monoid<W>
) => <A, B>(f: (a: A) => Writer<W, B>) => (as: readonly A[]) => Writer<W, readonly B[]>
/**
 * Equivalent to `ReadonlyArray#sequence(getApplicative(M))`.
 *
 * @since 3.0.0
 */
export declare const sequenceReadonlyArray: <W>(
  M: Monoid<W>
) => <A>(arr: readonly Writer<W, A>[]) => Writer<W, readonly A[]>
